"""
    | **Session**

        | aws.session.create()

    | **Properties**
    
        | { 
        |     'Name' = 'string',
        |     'CidrBlock' = 'string',
        |     'AmazonProvidedIpv6CidrBlock' = True|False,
        |     'Ipv6Pool' = 'string',
        |     'Ipv6CidrBlock' = 'string',
        |     'Ipv4IpamPoolId' = 'string',
        |     'Ipv4NetmaskLength' = 123,
        |     'Ipv6IpamPoolId' = 'string',
        |     'Ipv6NetmaskLength' = 123,
        |     'InstanceTenancy' = 'default'|'dedicated'|'host',
        |     'Ipv6CidrBlockNetworkBorderGroup' = 'string',
        |     'Tags' = [
        |         {
        |             'Key': 'string',
        |             'Value': 'string'
        |         }
        |     ]
        |     'EnableDnsSupport': True|False,
        |     'EnableDnsHostnames': True|False
        | }
    
    | **Response** 

        | {
        |     'CidrBlock': 'string',
        |     'DhcpOptionsId': 'string',
        |     'State': 'pending'|'available',
        |     'VpcId': 'string',
        |     'OwnerId': 'string',
        |     'InstanceTenancy': 'default'|'dedicated'|'host',
        |     'Ipv6CidrBlockAssociationSet': [
        |         {
        |             'AssociationId': 'string',
        |             'Ipv6CidrBlock': 'string',
        |             'Ipv6CidrBlockState': {
        |                 'State': 'associating'|'associated'|'disassociating'|'disassociated'|'failing'|'failed',
        |                 'StatusMessage': 'string'
        |             },
        |             'NetworkBorderGroup': 'string',
        |             'Ipv6Pool': 'string'
        |         },
        |     ],
        |     'CidrBlockAssociationSet': [
        |         {
        |             'AssociationId': 'string',
        |             'CidrBlock': 'string',
        |             'CidrBlockState': {
        |                 'State': 'associating'|'associated'|'disassociating'|'disassociated'|'failing'|'failed',
        |                 'StatusMessage': 'string'
        |             }
        |         },
        |     ],
        |     'IsDefault': True|False,
        |     'Tags': [
        |         {
        |             'Key': 'string',
        |             'Value': 'string'
        |         }
        |     ]
        |     'EnableDnsSupport': True|False,
        |     'EnableDnsHostnames': True|False
        | }

"""

import copy

from . import paginate
from . import client

def parse_properties(properties):
    copied_properties = copy.deepcopy(properties)
    tags = copied_properties.get('Tags', [])
    tags.append({ 
        "Key": "Name", 
        "Value": copied_properties.pop('Name')
    })
    copied_properties['TagSpecifications'] = [
        { 
            "ResourceType": "vpc", 
            "Tags": tags
        }
    ]
    return copied_properties

def put_vpc_attributes(session, vpc_id, properties):
    responses = []
    if 'EnableDnsSupport' in properties:
        responses.append(client.create(session, 'ec2').modify_vpc_attribute(
            EnableDnsSupport = { 'Value': properties['EnableDnsSupport'] },
            VpcId = vpc_id
        ))
    if 'EnableDnsHostnames' in properties:
        responses.append(client.create(session, 'ec2').modify_vpc_attribute(
            EnableDnsHostnames = { 'Value': properties['EnableDnsHostnames'] },
            VpcId = vpc_id
        ))
    return responses

def create(session, properties):
    """Create a VPC with the Name tag."""
    parsed_properties = parse_properties(properties)
    create_response = client.create(session, 'ec2').create_vpc(**parsed_properties)
    put_vpc_attributes(session, create_response['Vpc']['VpcId'], parsed_properties)
    return read(session, properties)

def read(session, properties):
    """Read a VPC by the Name tag or return None."""
    describe_responses = paginate.next_token(
        client.create(session, 'ec2').describe_vpcs, 
        Filters = [{ 'Name': 'tag:Name', 'Values': [ properties['Name'] ]}]
    )
    vpcs = []
    for describe_response in describe_responses:
        vpcs.extend(describe_response['Vpcs'])
    if len(vpcs) > 1: 
        raise Exception("Found multiple VPCs with the same name.")
    if len(vpcs) < 1:
        return None
    response = vpcs[0]
    response['EnableDnsSupport'] = client.create(session, 'ec2').describe_vpc_attribute(
        Attribute = 'enableDnsSupport',
        VpcId = response['VpcId']
    )
    response['EnableDnsHostnames'] = client.create(session, 'ec2').describe_vpc_attribute(
        Attribute = 'enableDnsHostnames',
        VpcId = response['VpcId']
    )
    return response

def update(session, properties):
    """Update a VPC with the Name tag."""
    parsed_properties = parse_properties(properties)
    read_response = read(session, properties)
    put_vpc_attributes(session, read_response['VpcId'], parsed_properties)
    if 'InstanceTenancy' in parsed_properties:
        tenancy_response = client.create(session, 'ec2').modify_vpc_tenancy(
            VpcId = read_response['VpcId'],
            InstanceTenancy = parsed_properties['InstanceTenancy']
        )
        if tenancy_response == False:
            raise Exception("Error modify_vpc_tenancy failed")
    return read(session, properties)

def delete(session, properties):
    """Delete a VPC with the Name tag."""
    read_response = read(session, properties)
    if read_response == None:
        return None
    response = client.create(session, 'ec2').delete_vpc(
        VpcId = read_response['VpcId']
    )
    return read(session, properties)

def put(session, properties):
    """Create or Update a VPC with the Name tag."""
    read_response = read(session, properties)
    if read_response == None:
        response = create(session, properties)
    else:
        response = update(session, properties)
    return response



"""

    | **Properties**

        | {
        |     'aws_access_key_id' = 'string', 
        |     'aws_secret_access_key' = 'string', 
        |     'aws_session_token' = 'string', 
        |     'region_name' = 'string', 
        |     'botocore_session' = `botocore.session.Session <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html>`_, 
        |     'profile_name' = 'string'
        | }

"""

import boto3

def create(properties):
    """Create a Session to access the AWS Environment."""
    response = boto3.session.Session(
        aws_access_key_id = properties.get('aws_access_key_id', None), 
        aws_secret_access_key = properties.get('aws_secret_access_key', None),
        aws_session_token = properties.get('aws_session_token', None),
        region_name = properties.get('region_name', None),
        botocore_session = properties.get('botocore_session', None),
        profile_name = properties.get('profile_name', None),
    ) 
    return response




def next_token(function, *args, **kwargs):
    responses = []
    while True:
        response = function(*args, **kwargs)
        responses.append(response)
        if 'NextToken' in response:
            kwargs['NextToken'] = response['NextToken']
        else:
            break
    return responses





import boto3

def next_token(function, *args, **kwargs):
    responses = []
    while True:
        response = function(*args, **kwargs)
        if 'NextToken' in response:
            kwargs['NextToken'] = response['NextToken']
        else:
            break
    return responses

def read(session = None, properties = {}):
    if session = None:
        session = boto3.session.Session()
    client = session.client('ec2')
    describe_properties = {
        "Filters": [{ 'Name': 'tag:Name', 'Values': [ name ]}]
    }
    while True:
        describe_response = client.describe_vpcs(**describe_properties)
        if 'NextToken' in describe_response:
            describe_properties['NextToken'] = describe_response['NextToken']
        else:
            break
    if len(describe_response['Vpcs']) > 1: 
        raise Exception("Found multiple VPCs with the same name.")
    else if len(describe_response['Vpcs']) == 1:
        return describe_response['Vpcs'][0]
    else: 
        return None








        create_response = client.create_vpc(
            CidrBlock = cidr,
            TagSpecifications = [{'Tags': [{ 'Key': 'Name', 'Value': name }]}]
        )
    else:
        response = client.modify_vpc_attribute(
            EnableDnsHostnames = { 'Value': enable_dns_hostnames },
            EnableDnsSupport = { 'Value': enable_dns_support },
            VpcId = describe_response['Vpcs']['VpcId']
        )
    return   


    import boto3

    def _client(session):
        return session.client('ec2')

    def create(session, **kwargs):
        client = _client(session)
        vpc_response = client.create_vpc(
            CidrBlock = kwargs['CidrBlock'],
            TagSpecifications = [{
                'Tags': [{ 'Key': 'Name', 'Value': kwargs['Name'] }]
            }]
        )
        return

    def read(session, **kwargs):
        client = _client(session)
        describe_response = client.describe_vpcs(
            Filters = [{ 'Name': 'tag:Name', 'Values': [ kwargs['Name'] ] }]
        )
        if len(describe_response) > 1: raise Exception("Found multiple VPCs with the same name.")
        return 

    vpc_name = "vpc_iac_example"
    print("Reading VPC")
    describe_response = boto3.client('ec2').describe_vpcs(
        Filters = [{ 'Name': 'tag:Name', 'Values': [ vpc_name ] }]
    )
    if len(describe_response['Vpcs']) > 1: raise Exception("Found multiple vpcs with the name: " + vpc_name)
    print(json.dumps(describe_response['Vpcs'], indent = 2))

    hello world

    if len(describe_response['Vpcs']) == 0:
        print("Creating VPC")
        create_response = boto3.client('ec2').create_vpc(
            CidrBlock = '10.0.0.0/16',
            TagSpecifications = [{ 
                'ResourceType': 'vpc',
                'Tags': [ { 'Key': 'Name', 'Value': vpc_name } ]
            }]
        )
        print(json.dumps(create_response['Vpc'], indent = 2))
    else:
        print("Updating VPC")
        
        # Update

